@@ -41,6 +41,7 @@
 #include "appchooserdialog.h"
 #include "externalwindow.h"
 
+static GHashTable *handles;
 
 typedef struct {
   XdpImplAppChooser *impl;
@@ -59,6 +60,7 @@ app_dialog_handle_free (gpointer data)
 {
   AppDialogHandle *handle = data;
 
+  g_hash_table_remove (handles, handle->request->id);
   g_clear_object (&handle->external_parent);
   g_object_unref (handle->request);
   g_object_unref (handle->dialog);
@@ -206,6 +208,8 @@ handle_choose_application (XdpImplAppChooser *object,
   handle->dialog = g_object_ref (dialog);
   handle->external_parent = external_parent;
 
+  g_hash_table_insert (handles, handle->request->id, handle);
+
   g_signal_connect (request, "handle-close",
                     G_CALLBACK (handle_close), handle);
 
@@ -224,6 +228,35 @@ handle_choose_application (XdpImplAppChooser *object,
   return TRUE;
 }
 
+static gboolean
+handle_update_choices (XdpImplAppChooser *object,
+                       GDBusMethodInvocation *invocation,
+                       const char *arg_handle,
+                       const char **choices)
+{
+  AppDialogHandle *handle;
+  g_autofree char *a = NULL;
+
+  handle = g_hash_table_lookup (handles, arg_handle);
+
+  if (handle == NULL)
+    {
+      g_dbus_method_invocation_return_error (invocation,
+                                             XDG_DESKTOP_PORTAL_ERROR,
+                                             XDG_DESKTOP_PORTAL_ERROR_NOT_FOUND,
+                                             "Request not found");
+      return TRUE;
+    }
+
+  g_debug ("Updating choices: %s\n", a = g_strjoinv (", ", (char **)choices));
+
+  app_chooser_dialog_update_choices (APP_CHOOSER_DIALOG (handle->dialog), choices);
+
+  g_dbus_method_invocation_return_value (invocation, NULL);
+
+  return TRUE;
+}
+
 gboolean
 app_chooser_init (GDBusConnection *bus,
                   GError **error)
@@ -233,6 +266,7 @@ app_chooser_init (GDBusConnection *bus,
   helper = G_DBUS_INTERFACE_SKELETON (xdp_impl_app_chooser_skeleton_new ());
 
   g_signal_connect (helper, "handle-choose-application", G_CALLBACK (handle_choose_application), NULL);
+  g_signal_connect (helper, "handle-update-choices", G_CALLBACK (handle_update_choices), NULL);
 
   if (!g_dbus_interface_skeleton_export (helper,
                                          bus,
@@ -240,6 +274,8 @@ app_chooser_init (GDBusConnection *bus,
                                          error))
     return FALSE;
 
+  handles = g_hash_table_new (g_str_hash, g_str_equal);
+
   g_debug ("providing %s", g_dbus_interface_skeleton_get_info (helper)->name);
 
   return TRUE;
