@@ -216,7 +216,7 @@ show_permission_dialog (BackgroundHandle *handle)
   gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
                                             _("This might be for a legitimate reason, but the application "
                                               "has not provided one."
-                                              "\n\nNote that forcing an application to quite might cause data loss."));
+                                              "\n\nNote that forcing an application to quit might cause data loss."));
   gtk_dialog_add_button (GTK_DIALOG (dialog), _("Force quit"), FORBID);
   gtk_dialog_add_button (GTK_DIALOG (dialog), _("Allow"), ALLOW);
 
@@ -305,7 +305,7 @@ handle_notify_background (XdpImplBackground *object,
   g_variant_builder_init (&builder, G_VARIANT_TYPE_VARDICT);
   g_variant_builder_add (&builder, "{sv}", "title", g_variant_new_string (_("Background activity")));
 
-  body = g_strdup_printf (_("%s is running in the background."), arg_name);
+  body = g_strdup_printf (_("“%s” is running in the background"), arg_name);
   g_variant_builder_add (&builder, "{sv}", "body", g_variant_new_string (body));
   g_variant_builder_add (&builder, "{sv}", "default-action", g_variant_new_string ("show"));
 
