@@ -31,6 +31,7 @@
 #include <gio/gunixfdlist.h>
 #include <gio/gunixinputstream.h>
 #include <gio/gunixoutputstream.h>
+#include <gio/gdesktopappinfo.h>
 
 #include "xdg-desktop-portal-dbus.h"
 
@@ -259,8 +260,27 @@ print_file (int fd,
   g_autoptr(GUnixInputStream) istream = NULL;
   g_autoptr(GUnixOutputStream) ostream = NULL;
   int fd2;
+  g_autofree char *app_desktop_fullname = NULL;
+  g_autoptr (GDesktopAppInfo) app_info = NULL;
+  // ensures the app_name won't be NULL even if the app_id.desktop file doesn't exist
+  g_autofree char *app_name = g_strdup (app_id);
 
-  title = g_strdup_printf ("Document from %s", app_id);
+  if (!g_str_has_suffix (app_id, ".desktop"))
+    {
+      app_desktop_fullname = g_strconcat (app_id, ".desktop", NULL);
+    }
+  else
+    {
+      app_desktop_fullname = g_strdup (app_id);
+    }
+
+  app_info = g_desktop_app_info_new (app_desktop_fullname);
+  if (app_info)
+    {
+      app_name = g_desktop_app_info_get_locale_string (app_info, "Name");
+    }
+
+  title = g_strdup_printf ("Document from %s", app_name);
 
   if (!preview)
     job = gtk_print_job_new (title, printer, settings, page_setup);
@@ -283,7 +303,6 @@ print_file (int fd,
                               NULL,
                               error) == -1)
     return FALSE;
-
   if (job)
     {
       if (!gtk_print_job_set_source_file (job, filename, error))
@@ -294,13 +313,17 @@ print_file (int fd,
   else
     {
       if (!launch_preview (filename, title, settings, page_setup, error))
-        return FALSE;
+	{
+	  unlink (filename);
+	  return FALSE;
+	}
     }
 
   /* The file will be removed when the GtkPrintJob closes it (once the job is
    * complete).
    */
-  unlink (filename);
+  if (!preview)
+    unlink (filename);
 
   return TRUE;
 }
@@ -484,8 +507,14 @@ handle_print (XdpImplPrint *object,
   gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (fake_parent));
   gtk_window_set_modal (GTK_WINDOW (dialog), modal);
   gtk_print_unix_dialog_set_manual_capabilities (GTK_PRINT_UNIX_DIALOG (dialog),
-                                                 can_preview () ? GTK_PRINT_CAPABILITY_PREVIEW : 0);
-
+                                                 can_preview () ? GTK_PRINT_CAPABILITY_PREVIEW : 0 |
+						 GTK_PRINT_CAPABILITY_PAGE_SET |
+						 GTK_PRINT_CAPABILITY_COPIES |
+						 GTK_PRINT_CAPABILITY_COLLATE |
+						 GTK_PRINT_CAPABILITY_REVERSE |
+						 GTK_PRINT_CAPABILITY_SCALE |
+						 GTK_PRINT_CAPABILITY_NUMBER_UP
+						 );
   handle = g_new0 (PrintDialogHandle, 1);
   handle->impl = object;
   handle->invocation = invocation;
@@ -632,7 +661,14 @@ handle_prepare_print (XdpImplPrint *object,
   gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (fake_parent));
   gtk_window_set_modal (GTK_WINDOW (dialog), modal);
   gtk_print_unix_dialog_set_manual_capabilities (GTK_PRINT_UNIX_DIALOG (dialog),
-                                                 can_preview () ? GTK_PRINT_CAPABILITY_PREVIEW : 0);
+                                                 can_preview () ? GTK_PRINT_CAPABILITY_PREVIEW : 0 |
+						 GTK_PRINT_CAPABILITY_PAGE_SET |
+						 GTK_PRINT_CAPABILITY_COPIES |
+						 GTK_PRINT_CAPABILITY_COLLATE |
+						 GTK_PRINT_CAPABILITY_REVERSE |
+						 GTK_PRINT_CAPABILITY_SCALE |
+						 GTK_PRINT_CAPABILITY_NUMBER_UP
+						 );
   gtk_print_unix_dialog_set_embed_page_setup (GTK_PRINT_UNIX_DIALOG (dialog), TRUE);
   gtk_print_unix_dialog_set_settings (GTK_PRINT_UNIX_DIALOG (dialog), settings);
   gtk_print_unix_dialog_set_page_setup (GTK_PRINT_UNIX_DIALOG (dialog), page_setup);
